!Define the data types and read/writes them to disk. Also change l_max here.

module cmbtypes
use settings
implicit none
  integer, parameter :: num_cls  = 1 !scalar+ num pol
  integer, parameter :: num_matter_power = 49 !number of points computed in matter power spectrum
  real, parameter :: matter_power_minkh =  3.16227766E-03 !minimum value of k/h to store
  real, parameter :: matter_power_dlnkh = 0.143911568     !log spacing in k/h
  integer, parameter :: lmax = 1300, lmax_tensor = 150
  real, parameter :: cl_norm = 1e-10 !units for As

  integer, parameter :: norm_As=1, norm_amp_ratio=2, norm_2df = 3

  Type CMBParams
     real norm(1:num_norm)
      !These are fast parameters controling amplitudes, calibrations, etc.
     real InitPower(1:num_initpower)
      !These are fast paramters that need to be handled carefully
     !Now remaining (non-independent) parameters
     real omb, omc, omv, omnu, omk, omdm
     real ombh2, omch2, omnuh2, omdmh2
     real zre, nufrac
     real h, H0
     real w
     real reserved(5)
  
  end Type CMBParams

  Type CosmoTheory
     real Age, r10
     real SN_loglike, reserved(3)
     !The following quantities are normalized assuming unit initial curvature perturbation
     real cl(lmax,num_cls), cl_tensor(lmax_tensor,num_cls)
     real sigma_8
     real matter_power(num_matter_power)
  end Type CosmoTheory

  logical, parameter ::  Old_format  = .false.

contains

   subroutine WriteModel(i,CMB, T, like, mult)
    integer i
    real, intent(in), optional :: mult
    Type(CosmoTheory) T
    real like, amult
    Type(CMBParams) CMB
    real acl(lmax), matpow(1:num_matter_power)
    integer j 

    if (present(mult)) then
       amult = mult
    else
       amult = 1
    end if
    
    if (.not. old_format) then
       write(i) amult, num_matter_power, lmax,lmax_tensor
       write(i) T%SN_loglike, T%reserved
    end if

    write(i) like
    write(i) CMB

    matpow = T%matter_power/(2*pi)**3
    !For historical reasons save in file with different normalization.

    write(i) T%Age, T%r10, T%sigma_8, matpow
    write(i) T%cl(2:20,1:num_cls)
    do j=30,190,10 
     write(i) T%cl(j,1:num_cls)
    end do
    do j=200,lmax, 25
     write(i) T%cl(j,1:num_cls)
    end do

    write(i) T%cl_tensor(2:20,1:num_cls)
    do j=30,min(lmax_tensor,190),10 
     write(i) T%cl_tensor(j,1:num_cls)
    end do
    do j=200,lmax_tensor, 25
     write(i) T%cl_tensor(j,1:num_cls)
    end do
    call flush(i)

   end subroutine WriteModel

   
 subroutine ReadModel(i,CMB, T, mult, like, error)
    integer, intent(in) :: i
    integer, intent(out) :: error
    real, intent(out) :: mult
    Type(CosmoTheory) T
    real like
    Type(CMBParams) CMB
    real icl(lmax,1:num_cls)
    integer j,ind, ix(lmax)
    integer almax,almaxtensor, anumpowers

    error = 0

    if (.not. old_format) then
       read(i,end=100,err=100) mult,anumpowers,almax,almaxtensor
       if (almax > lmax) stop 'reading file with larger lmax'
       read(i) T%SN_loglike, T%reserved
    else
       mult = 1
       anumpowers = num_matter_power
       almax=lmax
       almaxtensor = lmax_tensor
    end if

    read(i,end = 100, err=100) like
 
    read(i) CMB
    read(i) T%Age, T%r10, T%sigma_8, T%matter_power(1:anumpowers)

    T%matter_power(1:anumpowers) = T%matter_power(1:anumpowers)*(2*pi)**3
    !For historical reasons save in file with different normalization.

    read(i) icl(1:19,1:num_cls)
    ind =1
    do j =2,20 
       ix(ind)=j
       ind=ind+1
    end do
    do j=30,190,10 
     read(i) icl(ind,num_cls)
     ix(ind) = j
     ind = ind + 1
    end do
    do j=200,almax, 25
     read(i) icl(ind,1:num_cls)
     ix(ind) = j
     ind = ind+1
    end do
    ind = ind-1
 
   call InterpCls(ix,icl(1,1), T%cl(1,1), ind, lmax)
    read(i) icl(1:19,1:num_cls)
    ind =1
    do j =2,20 
       ix(ind)=j
       ind=ind+1
    end do
    do j=30,min(190,almaxtensor),10 
     read(i) icl(ind,num_cls)
     ix(ind) = j
     ind = ind + 1
    end do
    do j=200,almaxtensor, 25
     read(i) icl(ind,1:num_cls)
     ix(ind) = j
     ind = ind+1
    end do
    ind = ind-1
    call InterpCls(ix,icl(1,1), T%cl_tensor(1,1), ind, lmax_tensor)

    return
100 error = 1

   end subroutine ReadModel


      subroutine InterpCls(l,iCl, all_Cl, n, almax)
      integer, intent(in) :: n, almax
      real, intent(in) :: iCl(n)
      integer l(n)
      real all_Cl(l(n))
   
      integer il,llo,lhi,xi
      real xl(n),  ddCl(n)

      real a0,b0,ho
      real inCl(n)



      do il=1,n
         inCl(il) = iCl(il)*l(il)**2
      end do
      
      xl = l
      call spline_real(xl,inCl,n,ddCl)
     
            llo=1
            do il=2,l(n)
               xi=il
               if ((xi > l(llo+1)).and.(llo < n)) then
                  llo=llo+1
               end if
               lhi=llo+1
               ho=l(lhi)-l(llo)
               a0=(xl(lhi)-xi)/ho
               b0=(xi-xl(llo))/ho
              
               all_Cl(il) = (a0*inCl(llo)+ b0*inCl(lhi)+((a0**3-a0)* ddCl(llo) &
                       +(b0**3-b0)*ddCl(lhi))*ho**2/6)/il**2
      
            end do

            all_Cl(l(n)+1:almax) = 0
  
      end subroutine InterpCls
  
  
   subroutine ClsFromTheoryData(T, CMB, Cls)
     Type(CosmoTheory) T
     Type(CMBParams) CMB
     real Cls(lmax)

     Cls(2:lmax) = CMB%norm(norm_As)*T%cl(2:lmax,1)

     if (CMB%norm(norm_amp_ratio) /= 0) then
        Cls(2:lmax_tensor) =  Cls(2:lmax_tensor)+ CMB%norm(norm_As)*CMB%norm(norm_amp_ratio)*T%cl_tensor(2:lmax_tensor,1)
     end if 

   end subroutine ClsFromTheoryData

   subroutine WriteTextCls(aname,T, CMB)
     Type(CosmoTheory) T
     Type(CMBParams) CMB
     character (LEN=*), intent(in) :: aname
     integer l
     real Cls(lmax)

     call ClsFromTheoryData(T,CMB,Cls)
     open(unit = tmp_file_unit, file = aname, form='formatted', status = 'replace')
     do l=2, lmax
        write (tmp_file_unit,*) l, Cls(l)*l*(l+1)/(2*pi)
     end do
     close(tmp_file_unit)

   end subroutine WriteTextCls

   function MatterPowerAt(T,kh)
     !get matter power spectrum at kh = k/h by interpolation from stored values
     real, intent(in) :: kh
     Type(CosmoTheory) T
     real MatterPowerAt
     real x, d
     integer i
   
     x = log(kh/matter_power_minkh) / matter_power_dlnkh
     if (x < 0 .or. x >= num_matter_power) then
        write (*,*) ' k/h out of bounds in MatterPowerAt (',kh,')'
        stop
     end if
     i = int(x)
     d = x - i
     MatterPowerAt = exp(log(T%matter_power(i+1))*(1-d) + log(T%matter_power(i+2))*d)
     !Just do linear interpolation in logs for now..
     !(since we already cublic-spline interpolated to get the stored values)

   end function


end module cmbtypes
