!Parameterization where Omega_Lambda is derived and H_0 is parameter 3.

!Mapping between array of power spectrum parameters and CAMB
     subroutine SetCAMBInitPower(P,arr,in)
       use camb
       implicit none
       type(CAMBParams)  P
       integer, intent(in) :: in
       real arr(*)
   
       if (Power_Name /= 'power_tilt') stop 'Wrong initial power spectrum'
       P%InitPower%an(in) = arr(1)
       P%InitPower%ant(in) = arr(2)
       P%InitPower%rat(in) = 1
!       P%InitPower%n_run(in) = arr(3)

    end subroutine SetCAMBInitPower
 

 module param_type
   logical, parameter :: H0_derived  = .false.
   logical, parameter :: flat = .true. 
    !whether H0 is a fundamental or derived parameter
    !If flat and H0 not derived then derive Omlam

 end module param_type

 subroutine ParamsToCMBParams(Params, CMB)
     use settings
     use cmbtypes
     use LambdaGeneral
     use param_type
     implicit none
     real Params(num_Params)
     Type(CMBParams) CMB
     real h2

  
    CMB%ombh2 = Params(1)
    CMB%omdmh2 = Params(2)

    if (H0_derived) then
     CMB%omv = Params(3) 
    else
     CMB%H0 = Params(3)
    end if

    CMB%zre = Params(4) 
    CMB%Omk = Params(5)
    CMB%nufrac = Params(6)
    CMB%w = Params(7)

    CMB%InitPower(1:num_initpower) = Params(index_initpower:index_initpower+num_initPower-1)
    CMB%norm(1:num_norm) = Params(index_norm:index_norm+num_norm-1)

    if (H0_derived) then
     h2 = (CMB%omdmh2+CMB%ombh2)/(1-CMB%omv-CMB%omk)
     if (h2<0) then
      CMB%h = -1 !Obviously, this should be excluded by H0 prior
     else 
      CMB%h = sqrt(h2)
     end if
     CMB%H0 = CMB%h*100
    else
     CMB%h = CMB%H0/100
     h2 = CMB%h**2
    end if
    CMB%omnuh2 = CMB%omdmh2*CMB%nufrac
    CMB%omch2 = CMB%omdmh2 - CMB%omnuh2
    CMB%omb = CMB%ombh2/h2
    CMB%omc = CMB%omch2/h2
    CMB%omnu = CMB%omnuh2/h2
    CMB%omdm = CMB%omdmh2/h2
    if (.not. H0_derived) then
      CMB%omv = 1- CMB%omk - CMB%omb - CMB%omdm
    end if

   end subroutine ParamsToCMBParams

   subroutine CMBParamsToParams(CMB, Params)
     use settings
     use cmbtypes
     use LambdaGeneral
     use param_type
     implicit none
     real Params(num_Params)
     Type(CMBParams) CMB

 
      Params(1) =CMB%ombh2 
      Params(2) =CMB%omdmh2
 
      if (H0_derived) then
        Params(3) =CMB%omv 
      else
        Params(3) = CMB%H0
      end if
      Params(4) =CMB%zre
      Params(5) =CMB%omk 
      
      Params(6) =CMB%nufrac 
      Params(7) =CMB%w

      Params(index_initpower:index_initpower+num_initpower-1) =CMB%InitPower(1:num_initpower) 
      Params(index_norm:index_norm+num_norm-1) = CMB%norm(1:num_norm)
  
   end subroutine CMBParamsToParams


  subroutine WriteParams(P, mult, like)
     use settings
     use cmbtypes
     use ParamDef
     use param_type
     implicit none
    Type(ParamSet) P
    real, intent(in) :: mult, like
    character(LEN =30) fmt
    Type(CMBParams) C
    real r10
  
    integer l

    if (outfile_unit ==0) return
    call ParamsToCMBParams(P%P,C)

      if (lmax_tensor /= 0) then
          r10 = P%Info%Theory%cl_tensor(10,1)/P%Info%Theory%cl(10,1)*C%norm(norm_amp_ratio)
      else
        r10 = 0
      end if

      fmt = trim(numcat('(2E15.5,',num_params))//'E15.5,6E15.5)'
     if (H0_derived) then
       write (outfile_unit,fmt) mult,like, P%P, C%H0,P%Info%Theory%Age, C%omdm+C%omb, &
          P%Info%Theory%Sigma_8*sqrt(cl_norm*C%norm(norm_AS)),GetOpticalDepth(C),r10
     else
      write (outfile_unit,fmt) mult,like, P%P, C%omv,P%Info%Theory%Age, C%omdm+C%omb, &
          P%Info%Theory%Sigma_8*sqrt(cl_norm*C%norm(norm_AS)), GetOpticalDepth(C),r10
     end if
     call flush(outfile_unit)

  end  subroutine WriteParams
