! This module does post-processing of .data files. For example to importance sample new data,
! correct approximate theory (eg. add in lensing), or to compute missing theory (e.g. matter power).

module posthoc
  use settings
  use cmbtypes
  use CMB_Cls
  use cmbdata
  use CalcLike
  implicit none

  Type TPostParams
     logical  redo_like, redo_theory, redo_clsonly, redo_matteronly, redo_lensed
     integer redo_skip, redo_thin
     character(LEN=120) :: redo_datafile, redo_outroot
     real redo_likeoffset 
  end Type TPostParams

  Type(TPostParams) :: PostParams

contains

  subroutine ReadPostParams

    Ini_Fail_On_Not_Found = .false.
    PostParams%redo_like = Ini_Read_Logical('redo_likelihoods')
    PostParams%redo_theory = Ini_read_Logical('redo_theory')
    PostParams%redo_clsonly= Ini_read_Logical('redo_clsonly')
    PostParams%redo_matteronly= Ini_read_Logical('redo_matteronly')
    PostParams%redo_lensed= Ini_read_Logical('redo_lensed', .false.)
    PostParams%redo_skip = Ini_Read_Int('redo_skip',100)
    PostParams%redo_thin = max(1,Ini_Read_Int('redo_thin',1))
    PostParams%redo_datafile = Ini_Read_String('redo_datafile')
    PostParams%redo_outroot = Ini_Read_String('redo_outroot')
    PostParams%redo_likeoffset = Ini_Read_Double('redo_likeoffset',0.d0)
    if (PostParams%redo_lensed .and. (PostParams%redo_matteronly .or. PostParams%redo_clsonly)) &
          stop 'You can only redo lensing on its own'
        !easy to change of course..
  

  end subroutine ReadPostParams

   subroutine postprocess(InputFile)
        character(LEN=*), intent(INOUT):: InputFile
        Type(CMBParams) LastCMB,CMB, newCMB
        Type(CosmoTheory) Theory, CorrectTheory
        Type(ParamSetInfo) Info
        real Cls(lmax), Prior, truelike,mult,like
        real pow1, pow2,weight_min, weight_max, mult_sum, mult_ratio, mult_max,weight
        integer error,l, num, debug
        character (LEN=120) :: post_root


        weight_min= 1e30
        weight_max = -1e30
        mult_sum = 0
        mult_ratio = 0
        mult_max = -1e30

        debug = 0
        Info%Theory%Sn_LogLike = 0

        outfile_unit = 2

        if (PostParams%redo_datafile /= '') InputFile = PostParams%redo_datafile

        open(unit =1, file = trim(InputFile)//'.data', form='unformatted',status='old', err = 100)
        
        if (PostParams%redo_outroot == '') then
          post_root = trim(ExtractFilePath(InputFile))//'true_'// trim(ExtractFileName(InputFile))
        else
          post_root = PostParams%redo_outroot
          if (instance /= 0) post_root = numcat(trim(post_root)//'_',instance)
        end if
        if (Feedback > 0) then
           write (*,*) 'reading from: ' //  trim(InputFile)//'.data'
           write (*,*) 'writing to: ' // trim(post_root)//'.data'
        end if
  
        open(unit=outfile_unit, file = trim(post_root)//'.txt',form='formatted',status = 'replace')
        open(unit=3, file = trim(post_root)//'.data',form='unformatted',status = 'replace')
     
        num = 0
        do
        
        call ReadModel(1,CMB,Theory,mult,like, error)
           
        if (error ==1) exit

        num=num+1
        if (num>PostParams%redo_skip .and. mod(num,PostParams%redo_thin) == 0) then
     
        LastCMB = CMB

        newCMB = CMB  
        if (PostParams%redo_theory) then
         
         if (PostParams%redo_lensed) then
          call GetLensedCls(newCMB, CorrectTheory,error)
         else
          call GetClsInfo(newCMB, CorrectTheory, error, PostParams%redo_matteronly, PostParams%redo_clsonly)
         end if
        
         if (PostParams%redo_clsonly .and. .not. PostParams%redo_matteronly .or.PostParams%redo_lensed ) then
           
            Theory%cl = CorrectTheory%cl
            CorrectTheory = Theory
         end if
         if (PostParams%redo_matteronly) then
            Theory%sigma_8 = CorrectTheory%sigma_8
            Theory%Matter_Power = CorrectTheory%Matter_Power
            CorrectTheory = Theory
         end if
        else
          CorrectTheory = Theory
          error = 0
        end if
           
         if (error ==0) then
 
           if (PostParams%redo_like) then
           
              call ClsFromTheoryData(CorrectTheory, newCMB, Cls)
              Info%Theory = CorrectTheory
            
              if (any(Cls < 0)) then
                 write (*,*) 'WARNING: bad model with C_l < 0 being rejected'
                 write (*,*) 'in '//trim(InputFile)
                 !This shouldn't happen. 
                 !But good sanity check, esp when playing around with extended models
                 !Think have fixed problems with Om_k \sim -7e-4, w\sim -0.7.
                 write (*,*) newCMB
                 write (*,*)
                 truelike = logZero
              else
               truelike = GetLogLikePost(newCMB, Info, Cls)
              end if
              if (truelike == logZero) then
               weight = 0
              else
               weight = exp(like-truelike+PostParams%redo_likeoffset)
              end if
      
              mult = mult*weight
           else
              truelike = like
              weight = 1
           end if

           mult_ratio = mult_ratio + weight
           mult_sum = mult_sum + mult
          
           if (mult /= 0) then
        
            call WriteCMBParams(newCMB, CorrectTheory, mult, truelike)
            call WriteModel(3, newCMB, CorrectTheory,truelike,mult)
           end if
          
           if (Feedback > 1) write (*,*) 'done ', num, 'mult= ', mult,' weight = ', weight
           weight_max = max(weight,weight_max)       
           weight_min = min(weight,weight_min)
           mult_max = max(mult_max,mult)

        end if
        end if
        
        end do

        close(1)
        close(3)
        close(outfile_unit)

        num = (num - PostParams%redo_skip) / PostParams%redo_thin
        if (Feedback>0) then 
           write(*,*) 'finished. Processed ',num,' models'
           write (*,*) 'max weight= ',weight_max, ' min weight = ',weight_min
           write (*,*) 'mean mult  = ', mult_sum/num
           write (*,*) 'mean importance weight (approx evidence ratio) = ',mult_ratio/num
           write (*,*) 'effective number of samples =',mult_sum/mult_max
        end if
        
        if (mult_ratio < 1e-6) write (*,*) 'WARNING: use redo_likeoffset to rescale likelihoods'

        return

100     write (*,*) '.data file not found'
        stop

        end subroutine postprocess

end module posthoc
