module settings
  use AMLutils
  use Random
  use IniFile
  implicit none

  integer :: Feedback = 1
  integer :: num_threads = 0
  integer :: instance = 0
 
!num_norm is number of calibration and normalization parameters (fast)
!num_initpower is the number of initial power parameters
!num_hard is the number of `hard' parameters

  integer, parameter :: num_hard =7
  integer, parameter :: num_initpower = 3
  integer, parameter :: num_norm = 3

  integer, parameter :: num_params = num_norm + num_initpower + num_hard
  integer, parameter :: index_initpower = num_hard+1
  integer, parameter :: index_norm = index_initpower + num_initpower

  logical :: Use_LSS = .true.

  integer :: logfile_unit  = 0
  integer :: outfile_unit = 0 
  integer :: indepfile_unit = 0
  real, parameter :: logZero = 1e30
  character (LEN =120) :: FileChangeIni = ''


contains


 subroutine CheckParamChange
   logical bad, doexit

   if (FileChangeIni /= '') then

       call Ini_Open(FileChangeIni, tmp_file_unit, bad, .false.)
       if (bad) return
       Ini_fail_on_not_found = .false.
       doexit = (Ini_Read_Int('exit',0) == 1) 
       FeedBack = Ini_Read_Int('feedback',Feedback)
       num_threads = Ini_Read_Int('num_threads',num_threads)
       call Ini_Close
       call DeleteFile(FileChangeini)
       if (doexit) stop

   end if

 end subroutine CheckParamChange


 subroutine ReadMatrix(aname, mat, m,n)
   character(LEN=*), intent(IN) :: aname
   integer, intent(in) :: m,n
   real, intent(out) :: mat(m,n)
   integer j,k
   real tmp

   if (Feedback > 0) write(*,*) 'reading: '//trim(aname)
   call OpenTxtFile(aname, tmp_file_unit)

   do j=1,m
      read (tmp_file_unit,*, end = 200, err=100) mat(j,1:n)
   end do
   goto 120

100 rewind(tmp_file_unit)  !Try other possible format
   do j=1,m 
    do k=1,n
      read (tmp_file_unit,*, end = 200) mat(j,k)
    end do
   end do

120 read (tmp_file_unit,*, err = 150, end =150) tmp
   goto 200

150 close(tmp_file_unit)
    return

 200 write (*,*) 'matrix file '//trim(aname)//' is the wrong size'
     stop

 end subroutine ReadMatrix

  subroutine ReadVector(aname, vec, n)
   character(LEN=*), intent(IN) :: aname
   integer, intent(in) :: n
   real, intent(out) :: vec(n)
   integer j
   real tmp

   if (Feedback > 0) write(*,*) 'reading: '//trim(aname)

   call OpenTxtFile(aname, tmp_file_unit)

   do j=1,n
      read (tmp_file_unit,*, end = 200) vec(j)
   end do


    close(tmp_file_unit)
    return

 200 write (*,*) 'vector file '//trim(aname)//' is the wrong size'
     stop

 end subroutine ReadVector

 subroutine WriteVector(aname, vec, n)
   character(LEN=*), intent(IN) :: aname
   integer, intent(in) :: n
   real, intent(in) :: vec(n)
   integer j
  
   call CreateTxtFile(aname, tmp_file_unit)

   do j=1,n
      write (tmp_file_unit,'(1E15.5)') vec(j)
   end do

   close(tmp_file_unit)
  
 end subroutine WriteVector

 subroutine WriteSqMatrix(aname, mat,n)
   character(LEN=*), intent(in) :: aname
   integer, intent(in) :: n
   real, intent(in) :: mat(n,n)
   integer i
   character(LEN=50) fmt

   call CreateTxtFile(aname, tmp_file_unit)
   fmt = trim(numcat('(',n))//'E15.5)'
   do i=1, n
      write (tmp_file_unit, fmt) mat(i,1:n)
   end do

   close(tmp_file_unit)

 end subroutine WriteSqMatrix

       subroutine Diagonalize(m, diag, n)
          !Does m = U diag U^T, returning U in m
          integer, intent(in) :: n
          real m(n,n), diag(n)
          integer ierr, tmpsize
          real tmp(3*n**2)

          tmpsize = 3*n**2
          call SSYEV('V','U',n,m,n,diag,tmp,tmpsize,ierr) !evalues and vectors of symmetric matrix
        
       end subroutine Diagonalize


end module settings

