!Use supernova cosmology project results
!See astro-ph/9812133 and www-supernova.lbl.gov
!Using only SN in their `fit C'

!Marginalize anayltically over M_b with flat prior - 
!resultant log likelihood has arbitary origin, but is returned equal
!to the best-fit value.
module snovae
use cmbtypes
implicit none

 integer, parameter :: SN_num = 54
 real :: SN_z(SN_num), SN_mbeff(SN_num), SN_diagerr(SN_num),SN_Ninv(SN_num,SN_Num), SN_Ninvmarge(SN_num,SN_Num)
 real :: SN_trNinv
 logical :: do_SN_init = .true.
contains


 subroutine SN_init
   character (LEN=1200) :: InLine
   character (LEN=20) :: names(60)
   real inarr(9)
   real input (60,3), covmatrix(60,60), tmp(SN_num)
   integer fitc(SN_num)
   integer i,j, x, y

   if (Feedback > 0) write (*,*) 'reading: supernovae data'
   call OpenTxtFile('data/sn_data.dat',tmp_file_unit)
   read(tmp_file_unit,'(a)') InLine
   do i=1, 60
      read(tmp_file_unit, *) names(i), input(i,:)
   end do
   close(tmp_file_unit)

!Use only those supernovae used for `fit c'
   fitc =  (/1, (I, I=3, 17),19,20,21,22, (I,I=24,30), (I,I=32,53), 55, 57,58,59,60/)

   SN_z = input(fitc,1)
   SN_mbeff = input(fitc,2)
   SN_diagerr = 1/input(fitc, 3)**2
   SN_trNinv = sum(SN_diagerr)

!   call OpenTxtFile('data/sn_cov_matrix.dat',tmp_file_unit)
!   do i=1,60
!    do j=1,60
!       read(tmp_file_unit,*) x, y, covmatrix(x,y)
!    end do
!   end do
!   close(tmp_file_unit)
!   SN_Ninv = covmatrix(fitc,fitc)

!Get matrix after analytic marginalization over M_b
!   tmp = 1
!   tmp = MATMUL(SN_Ninv,tmp)/sqrt(sum(SN_Ninv))
!   do i=1,SN_num
!    do j=1,SN_num
!       SN_Ninvmarge(i,j) = SN_Ninv(i,j) - tmp(i)*tmp(j)
!    end do
!   end do

   do_SN_init = .false.

 end subroutine SN_init

 function SN_LnLike(CMB)
  !Assume this is called just after CAMB with the correct model
  use camb
  type(CMBParams) CMB
  real SN_LnLike
  integer i
  double precision z
  real MB, diffs(SN_num), chisq

  if (do_SN_init) call SN_init

  !$OMP PARALLEL DO DEFAULT(SHARED),SCHEDULE(STATIC), PRIVATE(z,i)
  do i=1, SN_num
   !Obviously this is not v efficient...
    z= SN_z(i)
    diffs(i) = 5*log10(CMB%H0*(1+z)**2*AngularDiameterDistance(z)) - SN_mbeff(i)
  end do
  !$OMP END PARALLEL DO 
  
!   chisq = sum(diffs*matmul(SN_Ninvmarge,diffs)) 
   chisq = (sum(diffs**2*SN_diagerr) - sum(SN_diagerr*diffs)**2/SN_trNinv )* 50./54
   if (Feedback > 1) write (*,*) 'SN chisq: ', chisq
   SN_LnLike = chisq/2

 end function SN_LnLike



end module snovae
